/* 
 * Stellarium
 * Copyright (C) 2002 Fabien Chreau
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "s_gui_window.h"

void ExtStdWinClicCallback(int x,int y,enum guiValue state,enum guiValue button,Component * me)
{	((StdWin*)me)->StdWinClicCallback(button, state);
	((Container *)me)->handleMouseClic((int)(x-me->getPosition()[0]),(int)(y-me->getPosition()[1]),state,button);
}

void ExtStdWinMoveCallback(int x,int y,enum guiValue action,Component * me)
{	((StdWin*)me)->StdWinMoveCallback(x,y,action);
	((Container *)me)->handleMouseMove((int)(x-me->getPosition()[0]),(int)(y-me->getPosition()[1]));
}

StdWin::StdWin(float posx, float posy, float sizex, float sizey, char * _title, Component * parent)
 : Container(), mouseOn(false)
{	setTitle(_title);
	reshape(vec2_t(posx,posy),vec2_t(sizex,sizey));
	setClicCallback(ExtStdWinClicCallback);
	setMoveCallback(ExtStdWinMoveCallback);
}


void StdWin::StdWinClicCallback(enum guiValue button,enum guiValue state)
{	if (state==GUI_DOWN)
	{	mouseOn=true;
		if (overBar) draging=true;
	}
	else
	{	if (state==GUI_UP) mouseOn=false;
		draging=false;
	}
}

void StdWin::StdWinMoveCallback(int x, int y,enum guiValue action)
{	if (action==GUI_MOUSE_ENTER)
	{	mouseOn=false;
	}
	if (action==GUI_NOTHING && !mouseOn)
	{	lastXY[0]=x;
		lastXY[1]=y;
	}
	if (y<=position[1]+headerSize)
	{	overBar=true;
	}
	else overBar=false;
	if (draging)
	{	position[0]+=x-lastXY[0];
		position[1]+=y-lastXY[1];
		lastXY[0]=x;
		lastXY[1]=y;
	}
}

void StdWin::render(GraphicsContext& gc)
{   vec2_t pos = getPosition();
    vec2_t sz = getSize();
	headerSize = gc.getFont()->getLineHeight()+2;
	glColor3f(gc.baseColor[0],gc.baseColor[1],gc.baseColor[2]);
  	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);
	glBindTexture(GL_TEXTURE_2D,gc.backGroundTexture);
	glBegin(GL_QUADS );
		glTexCoord2f(0.0f,0.0f);	glVertex3f(pos[0],        pos[1] + sz[1],        0);	// Bas Gauche
		glTexCoord2f(1.0f,0.0f);    glVertex3f(pos[0] + sz[0], pos[1] + sz[1],        0);	// Bas Droite
		glTexCoord2f(1.0f,1.0f);    glVertex3f(pos[0] + sz[0], pos[1], 0);	// Haut Droit
		glTexCoord2f(0.0f,1.0f);    glVertex3f(pos[0],        pos[1], 0);	// Haut Gauche
	glEnd ();
	glBindTexture(GL_TEXTURE_2D,gc.headerTexture);
	glBegin(GL_QUADS );
		glTexCoord2f(0.0f,0.0f);	glVertex3f(pos[0],        pos[1],        0);	// Bas Gauche
		glTexCoord2f(1.0f,0.0f);    glVertex3f(pos[0] + sz[0], pos[1],        0);	// Bas Droite
		glTexCoord2f(1.0f,1.0f);    glVertex3f(pos[0] + sz[0], pos[1] + headerSize, 0);	// Haut Droit
		glTexCoord2f(0.0f,1.0f);    glVertex3f(pos[0],        pos[1] + headerSize, 0);	// Haut Gauche
	glEnd ();
	glDisable(GL_BLEND);
	glDisable(GL_TEXTURE_2D);

    glBegin(GL_LINE_LOOP);
    glVertex3f(pos[0],        pos[1],        0);
    glVertex3f(pos[0] + sz[0], pos[1],        0);
    glVertex3f(pos[0] + sz[0], pos[1] + sz[1], 0);
    glVertex3f(pos[0],        pos[1] + sz[1], 0);
    glEnd();

    glBegin(GL_LINES);
    glVertex3f(pos[0],        pos[1]+headerSize,        0);
    glVertex3f(pos[0] + sz[0], pos[1]+headerSize,        0);
    glEnd();

	glColor3f(gc.textColor[0],gc.textColor[1],gc.textColor[2]);
	glEnable(GL_BLEND);
    glEnable(GL_TEXTURE_2D);
    glPushMatrix();
    glTranslatef(pos[0] + (sz[0] - gc.getFont()->getStrLen(title)) / 2,
                 pos[1] + (headerSize - gc.getFont()->getLineHeight()) / 2,
                 0);
    gc.getFont()->print(0,0,title);
    glPopMatrix();
	Container::render(gc);
}

const char * StdWin::getTitle() const
{
    return title;
}

void StdWin::setTitle(char * _title)
{	strncpy(title,_title,127);
}


/**** StdBtWin ***/
void closeBtOnClicCallback(guiValue button,Component * me)
{	((StdBtWin*)me->getParent())->Hide();
}

StdBtWin::StdBtWin(float posx, float posy, 
					float sizex, float sizey,
					char * title, Component * parent)
					: StdWin(posx,posy,sizex,sizey,title,parent), onHideCallback(NULL)
{	Button * closeBt = new Button();
	vec2_t P(size[0]-12,2);
	vec2_t V(10,10);
	closeBt->reshape(P,V);
	closeBt->setOnClicCallback(closeBtOnClicCallback);
	addComponent(closeBt);
}

void StdBtWin::Hide(void)
{	if (onHideCallback!=NULL) onHideCallback();
}
