/* 
 * Stellarium
 * Copyright (C) 2002 Fabien Chreau
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

// class used to manage groups of Star

#include "star_mgr.h"
#include "texture.h"

extern double precalculatedFOVStar;

Star_mgr::Star_mgr()
{
}

// Load from file ( create the stream and call the Read function )
void Star_mgr::Load(char * NameFichier)
{	FILE * pFile;
	pFile=fopen(NameFichier,"r");
	if (pFile==NULL)
	{	printf("ERREUR FICHIER STARS\n");
		return;
	}
	Read(pFile);
	fclose(pFile);
}

// Destructor
Star_mgr::~Star_mgr()
{	vector<Star *>::iterator iter;
	for(iter=Liste.begin();iter!=Liste.end();iter++)
	{	delete (*iter);
	}
}

// read from stream
int Star_mgr::Read(FILE * catalog)
{	char ligne[200];
	char strTmp[5];
	char strTmp2[200];
	FILE * pFile;
	pFile=fopen(DATA_DIR "/name.txt","rt");
	if (pFile==NULL)
	{	printf("ERROR : " DATA_DIR "/name.txt is an invalid file");
		return 1;
	}
	fgets(ligne,200,pFile);
	ligne[199]='\0';
	strncpy(strTmp,ligne,4);	// HR number
	strTmp[4]='\0';
	unsigned int numHR = atoi(strTmp);
	strcpy(strTmp2,ligne+4);
	strTmp2[strlen(strTmp2)-1]='\0';

	printf("Loading star data...\n");
 	for(int i=1;i<=9110;i++)
	{	Star * e = new Star;
		(*e).Read(catalog);
		Liste.push_back(e);
		if (numHR==(*e).HR)
		{	// assign the common name
			strcpy((*e).CommonName,strTmp2);
			// take the next one
			fgets(ligne,200,pFile);
			ligne[199]='\0';
			strncpy(strTmp,ligne,4);	// HR number
			strTmp[4]='\0';
			numHR = atoi(strTmp);
			strcpy(strTmp2,ligne+4);
			strTmp2[strlen(strTmp2)-1]='\0';
		}
		else (*e).CommonName[0]=0;
	}
	printf("\n");
	fclose(pFile);
	Star::starTextureId=createTexture("etoile16x16Aigrettes");	// Load star texture
	return 0;
}

// Draw all the stars
void Star_mgr::Draw(void)
{	precalculatedFOVStar=((global.Fov+4.)*PI*((double)global.X_Resolution/global.Y_Resolution)/360.); // Precalc the field of view for optimizations
	glBindTexture (GL_TEXTURE_2D, Star::starTextureId);
	glEnable(GL_TEXTURE_2D);
	glEnable(GL_BLEND);

	double z;
	glColor3f(0.5, 1.0, 0.5);
	GLdouble M[16];
	GLdouble P[16];
	GLint V[4];
	glGetDoublev(GL_MODELVIEW_MATRIX,M);
	glGetDoublev(GL_PROJECTION_MATRIX,P);
	glGetIntegerv(GL_VIEWPORT,V);

	setOrthographicProjection(global.X_Resolution, global.Y_Resolution);
	glPushMatrix();
	glLoadIdentity();
	vector<Star *>::iterator iter;
	for(iter=Liste.begin();iter!=Liste.end();iter++)
	{	gluProject((**iter).XYZ[0],(**iter).XYZ[1],(**iter).XYZ[2],M,P,V,&((**iter).XY[0]),&((**iter).XY[1]),&z);
		if (z<1) (**iter).Draw();
	}
	glPopMatrix();
	resetPerspectiveProjection();
}

// Look for a star by XYZ coords
int Star_mgr::Rechercher(vec3_t Pos)
{	Pos.Normalize();
	vector<Star *>::iterator iter;
	Star * plusProche;
	float anglePlusProche=3.15;
	for(iter=Liste.begin();iter!=Liste.end();iter++)
	{	if ((**iter).XYZ[0]*Pos[0]+(**iter).XYZ[1]*Pos[1]+(**iter).XYZ[2]*Pos[2]>anglePlusProche)
		{	anglePlusProche=(**iter).XYZ[0]*Pos[0]+(**iter).XYZ[1]*Pos[1]+(**iter).XYZ[2]*Pos[2];
			plusProche=(*iter);
		}
	}
//	printf("PlusProche : Name:%s HR:%i RA:%4.2f DE:%4.2f Dist:%4.5f\n",&(*plusProche).Name,(*plusProche).HR,(*plusProche).RaRad*12/3.141,(*plusProche).DecRad*180/3.141,anglePlusProche);
	if (anglePlusProche>498)
	{	Selectionnee=plusProche;
		return 0;
	}
	else return 1;
}

// Look for a star by Harvard Revised Catalog number
Star * Star_mgr::Rechercher(unsigned int HR)
{	return Liste[HR-1];
}
