/* 
 * Stellarium
 * Copyright (C) 2002 Fabien Chreau
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

#include "stellarium_ui.h"
#include "s_font.h"
#include "s_gui.h"
#include "texture.h"
#include "s_utility.h"
#include "s_gui_window.h"
#include "navigation.h"
#include "nebula_mgr.h"
#include "star_mgr.h"
#include "stellarium.h"

using namespace gui;

int firstTextureNB,lastTextureNB;

s_font * spaceFont;						// The font used in the GraphicContext (=Skin)
GraphicsContext * gc;					// The graphic context used to give the widget their drawing style
Container * Base;						// The container who contains everything (=the desktop)
Label * btLegend;						// The dynamic information about the button menu

Container * ContainerBtFlags;			// Flags button List
Textured_Button * BtConstellationsDraw;
Textured_Button * BtConstellationsName;
Textured_Button * BtAzimutalGrid;
Textured_Button * BtEquatorialGrid;
Textured_Button * BtGround;
Textured_Button * BtFog;
Textured_Button * BtRealTime;
Textured_Button * BtAcceleredTime;
Textured_Button * BtVeryFastTime;
Textured_Button * BtCardinalPoints;
Textured_Button * BtAtmosphere;
Textured_Button * BtNebula;
Textured_Button * BtHelp;
Textured_Button * BtRealMode;
Textured_Button * BtTracking;

TextLabel * HelpTextLabel;				// The TextLabel containing the help infos
StdBtWin * HelpWin;						// The window containing the help info

FilledTextLabel * InfoSelectLabel;			// The TextLabel displaying teh infos about the selected object

FilledContainer * TopWindowsInfos;		// The top bar containing the infos
Label * DateLabel;
Label * HourLabel;
Label * FPSLabel;
Label * AppNameLabel;
Label * FOVLabel;

// declaration of callbacks for buttons and windows
void BtFlagsOnClicCallBack(guiValue button,Component *);
void BtFlagsOnMouseOverCallBack(guiValue button,Component *);
void HelpWinHideCallback(void) {	global.FlagHelp=false;
									HelpWin->setVisible(false);
									BtHelp->setActive(false);
								}

/*******************************************************************************/
void initUi(void)
{	firstTextureNB=getCurrentTextNum()+1;
	/*** fonts ***/
	spaceFont = new s_font(0.013*global.X_Resolution, "spacefont", DATA_DIR "/spacefont.txt");

	/*** Colors ***/
	vec3_t clBlue(0.3,0.3,1.0);
	vec3_t clRed(1.0,0.3,0.3);
	vec3_t clGreen(0.3,1.0,0.3);
	vec3_t clWhite(0.7,0.7,0.7);
	
	/*** Graphic Context ***/
	gc = new GraphicsContext(global.X_Resolution,global.Y_Resolution);
	gc->setFont(spaceFont);
	gc->backGroundTexture=createTexture("backmenu");
	gc->headerTexture=createTexture("headermenu");

	/*** Flag Buttons list ***/
	float btSize=4*gc->getFont()->getAverageCharLen();
	float xpos=0;
	vec2_t pos(xpos,1);
	vec2_t btsz(btSize,btSize);
	vec2_t step(btSize,0);

	BtConstellationsDraw = new Textured_Button(createTexture("Bouton1"),pos,btsz,clBlue,clBlue/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,1,global.FlagConstellationDrawing);
	BtConstellationsName = new Textured_Button(createTexture("Bouton2"),pos+=step,btsz,clBlue,clBlue/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,2,global.FlagConstellationName);
	BtAzimutalGrid = new Textured_Button(createTexture("Bouton3"),pos+=step,btsz,clRed,clRed/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,3,global.FlagAzimutalGrid);
	BtEquatorialGrid = new Textured_Button(createTexture("Bouton3"),pos+=step,btsz,clRed,clRed/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,4,global.FlagEquatorialGrid);
	BtNebula = new Textured_Button(createTexture("Bouton12"),pos+=step,btsz,clWhite,clWhite/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,12,global.FlagNebula);
	BtGround = new Textured_Button(createTexture("Bouton4"),pos+=step,btsz,clWhite,clWhite/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,5,global.FlagGround);
	BtFog = new Textured_Button(createTexture("Bouton7"),pos+=step,btsz,clWhite,clWhite/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,6,global.FlagFog);
	BtRealTime = new Textured_Button(createTexture("Bouton5"),pos+=step,btsz,clWhite,clWhite/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,7,global.FlagRealTime);
	BtAcceleredTime = new Textured_Button(createTexture("Bouton6"),pos+=step,btsz,clWhite,clWhite/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,8,global.FlagAcceleredTime);
	BtVeryFastTime = new Textured_Button(createTexture("Bouton6"),pos+=step,btsz,clWhite,clWhite/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,9,global.FlagVeryFastTime);
	BtCardinalPoints = new Textured_Button(createTexture("Bouton8"),pos+=step,btsz,clRed,clRed/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,10,global.FlagCardinalPoints);
	BtAtmosphere = new Textured_Button(createTexture("Bouton9"),pos+=step,btsz,clWhite,clWhite/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,11,global.FlagAtmosphere);
	BtRealMode = new Textured_Button(createTexture("Bouton14"),pos+=step,btsz,clWhite,clWhite/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,14,global.FlagRealMode);
	BtHelp = new Textured_Button(createTexture("Bouton11"),pos+=step,btsz,clWhite,clWhite/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,13,global.FlagHelp);
	BtTracking = new Textured_Button(createTexture("Bouton13"),pos+=step,btsz,clWhite,clWhite/2,BtFlagsOnClicCallBack,BtFlagsOnMouseOverCallBack,15,global.FlagTracking);
	/*** Button container ***/
	ContainerBtFlags = new Container();
	ContainerBtFlags->reshape(vec2_t(0,global.Y_Resolution-btSize-2),	vec2_t(btSize*15+1,btSize+1));
	ContainerBtFlags->addComponent(BtConstellationsDraw);
	ContainerBtFlags->addComponent(BtConstellationsName);
	ContainerBtFlags->addComponent(BtAzimutalGrid);
	ContainerBtFlags->addComponent(BtEquatorialGrid);
	ContainerBtFlags->addComponent(BtGround);
	ContainerBtFlags->addComponent(BtFog);
	ContainerBtFlags->addComponent(BtRealTime);
	ContainerBtFlags->addComponent(BtAcceleredTime);
	ContainerBtFlags->addComponent(BtVeryFastTime);
	ContainerBtFlags->addComponent(BtCardinalPoints);
	ContainerBtFlags->addComponent(BtAtmosphere);
	ContainerBtFlags->addComponent(BtNebula);
	ContainerBtFlags->addComponent(BtHelp);
	ContainerBtFlags->addComponent(BtRealMode);
	ContainerBtFlags->addComponent(BtTracking);

	/*** button legend label ***/
	btLegend = new Label("ERROR, this shouldn't be seen...");
	btLegend->reshape(vec2_t(3,global.Y_Resolution-btSize-gc->getFont()->getLineHeight()-5),vec2_t(100,17));
	btLegend->setVisible(false);

	/*** help TextLabel ***/
	HelpTextLabel = new TextLabel(
		"4 Directions : Deplacement RA/DE\n\
		Page Up/Down : Zoom\n\
		Left Mouse Clic : Select Star\n\
		Right Mouse Clic  : Clear Pointer\n\
		SPACE or Middle Mouse Clic:\n\
		     Center On Selected Object\n\
		A   : Drawing of the Constellations\n\
		Z   : Names of the Constellations\n\
		E   : Equatorial Grid\n\
		R   : Azimutal Grid\n\
		T   : Nebulas\n\
		Y   : Ground\n\
		U   : Fog\n\
		I   : Real Time Mode\n\
		O   : Accelered Time Mode\n\
		P   : Very Fast Time Mode\n\
		Q   : Cardinal Points\n\
		S   : Atmosphere\n\
		D   : RealMode\n\
		H   : Help\n\
		K   : Ecliptic\n\
		J   : Equator\n\
		L   : Object Tracking\n\
		M   : Stars\n\
		+   : Inrease Sky Brigthness\n\
		-   : Decrease Sky Brigthness\n\
		ESC : Quit\n\
		F1/F2 : fullscreen window/small window\n\
		(in windowed mode only)\n"
	,gc->getFont());
	HelpTextLabel->reshape(vec2_t(gc->getFont()->getAverageCharLen()*5,gc->getFont()->getLineHeight()*2.5),vec2_t(45*gc->getFont()->getAverageCharLen(),32*gc->getFont()->getLineHeight()));

	HelpWin = new StdBtWin(global.X_Resolution/2-25*gc->getFont()->getAverageCharLen(), global.Y_Resolution/2-26*gc->getFont()->getLineHeight()/2, 48*gc->getFont()->getAverageCharLen(), 33*(gc->getFont()->getLineHeight()+1), "Help", Base);
	HelpWin->addComponent(HelpTextLabel);
	HelpWin->setVisible(global.FlagHelp);
	HelpWin->setHideCallback(HelpWinHideCallback);

	/*** InfoSelect TextLabel ***/
	InfoSelectLabel = new FilledTextLabel("Info",gc->getFont());
	InfoSelectLabel->reshape(vec2_t(3,gc->getFont()->getLineHeight()+2),vec2_t(gc->getFont()->getAverageCharLen()*50,7*gc->getFont()->getLineHeight()));
	InfoSelectLabel->setVisible(false);

	float avgCharLen=gc->getFont()->getAverageCharLen();
	float lineHeight=gc->getFont()->getLineHeight();
	/*** Date Label ***/
	DateLabel = new Label("-");
	DateLabel->reshape(vec2_t(3,2), vec2_t(10*avgCharLen,lineHeight));
	/*** Hour Label ***/
	HourLabel = new Label("-");
	HourLabel->reshape(vec2_t(10*avgCharLen+15,2), vec2_t(6*avgCharLen,lineHeight));
	/*** FPS Label ***/
	FPSLabel = new Label("-");
	FPSLabel->reshape(vec2_t(global.X_Resolution-11*avgCharLen,2), vec2_t(10*avgCharLen,lineHeight));

	if (!global.FlagFps) FPSLabel->setVisible(false);
	/*** FOV Label ***/
	FOVLabel = new Label("-");
	FOVLabel->reshape(vec2_t(global.X_Resolution-24*avgCharLen,2), vec2_t(10*avgCharLen,lineHeight));
	/* AppName Label */
	AppNameLabel = new Label(APP_NAME);
	AppNameLabel->reshape(vec2_t(global.X_Resolution/2-gc->getFont()->getStrLen(APP_NAME)/2,2), vec2_t(gc->getFont()->getStrLen(APP_NAME),lineHeight));

	/*** TopWindowsInfos Container ***/
	TopWindowsInfos = new FilledContainer();
	TopWindowsInfos->reshape(vec2_t(0,0), vec2_t(global.X_Resolution,gc->getFont()->getLineHeight()+2));
	TopWindowsInfos->addComponent(DateLabel);
	TopWindowsInfos->addComponent(HourLabel);
	TopWindowsInfos->addComponent(FPSLabel);
	TopWindowsInfos->addComponent(FOVLabel);
	TopWindowsInfos->addComponent(AppNameLabel);

	/*** Base container ***/
	Base = new Container();
	Base->reshape(vec2_t(0,0),vec2_t(global.X_Resolution,global.Y_Resolution));
	Base->addComponent(ContainerBtFlags);
	Base->addComponent(btLegend);
	Base->addComponent(InfoSelectLabel);
	Base->addComponent(TopWindowsInfos);
	Base->addComponent(HelpWin);

	lastTextureNB=getCurrentTextNum();
}

void clearUi(void)
{	delete Base;
	for (int i=lastTextureNB;i>=firstTextureNB;i--)
	deleteTexture(i);
}

/********************************************************************/
void BtFlagsOnClicCallBack(guiValue button,Component * bt)
{	switch (bt->getID())
	{	case 1 :	global.FlagConstellationDrawing=!global.FlagConstellationDrawing;
					bt->setActive(global.FlagConstellationDrawing);
					return;
		case 2 :	global.FlagConstellationName=!global.FlagConstellationName;
					bt->setActive(global.FlagConstellationName);
					return;
		case 3 :	global.FlagAzimutalGrid=!global.FlagAzimutalGrid;
					bt->setActive(global.FlagAzimutalGrid);
					return;
		case 4 :	global.FlagEquatorialGrid=!global.FlagEquatorialGrid;
					bt->setActive(global.FlagEquatorialGrid);
					return;
		case 5 :	global.FlagGround=!global.FlagGround;
					bt->setActive(global.FlagGround);
					return;
		case 6 :	global.FlagFog=!global.FlagFog;
					bt->setActive(global.FlagFog);
					return;
		case 7 :	global.FlagRealTime=!global.FlagRealTime;
					bt->setActive(global.FlagRealTime);
					return;
		case 8 :	global.FlagAcceleredTime=!global.FlagAcceleredTime;
					bt->setActive(global.FlagAcceleredTime);
					return;
		case 9 :	global.FlagVeryFastTime=!global.FlagVeryFastTime;
					bt->setActive(global.FlagVeryFastTime);
					return;
		case 10 :	global.FlagCardinalPoints=!global.FlagCardinalPoints;
					bt->setActive(global.FlagCardinalPoints);
					return;
		case 11 :	global.FlagAtmosphere=!global.FlagAtmosphere;
					bt->setActive(global.FlagAtmosphere);
					return;
		case 12 :	global.FlagNebula=!global.FlagNebula;
					bt->setActive(global.FlagNebula);
					return;
		case 13 :	global.FlagHelp=!global.FlagHelp;
					HelpWin->setVisible(global.FlagHelp);
					bt->setActive(global.FlagHelp);
					return;
		case 14 :	global.FlagRealMode=!global.FlagRealMode;
					bt->setActive(global.FlagRealMode);
					Base->setVisible(!global.FlagRealMode);
					break;
		case 15 :	global.FlagTracking=!global.FlagTracking;
					bt->setActive(global.FlagTracking);
					break;

					return;

	}
}

/*******************************************************************************/
void BtFlagsOnMouseOverCallBack(guiValue event,Component * bt)
{	if (event==GUI_MOUSE_LEAVE) btLegend->setVisible(false);
	else btLegend->setVisible(true);
	switch (bt->getID())
	{	case 1 : btLegend->setLabel("Drawing of the Constellations"); return;
		case 2 : btLegend->setLabel("Names of the Constellations"); return;
		case 3 : btLegend->setLabel("Azimutal Grid"); return;
		case 4 : btLegend->setLabel("Equatorial Grid"); return;
		case 5 : btLegend->setLabel("Ground"); return;
		case 6 : btLegend->setLabel("Fog"); return;
		case 7 : btLegend->setLabel("Real Time Mode"); return;
		case 8 : btLegend->setLabel("Accelered Time Mode"); return;
		case 9 : btLegend->setLabel("Very Fast Time Mode"); return;
		case 10 :btLegend->setLabel("Cardinal Points"); return;
		case 11 :btLegend->setLabel("Atmosphere (+ and - to control the brightness)"); return;
		case 12 :btLegend->setLabel("Nebulas"); return;
		case 13 :btLegend->setLabel("Help"); return;
		case 14 :btLegend->setLabel("Real Mode"); return;
		case 15 :btLegend->setLabel("Object Tracking"); return;
	}
}

/*******************************************************************************/
void updateStandardWidgets(void)
{	// Update the date and time
	char str[30];
	int jour,mois;
	long annee;
	DateOps::dayToDmy((long)global.JDay,jour,mois,annee);
	double reste=global.JDay-(int)DateOps::dmyToDay(jour,mois,annee);
	double heure=reste*24;
	double minute=(double)(heure-(int)heure)*60;
	double seconde=(minute-(int)minute)*60;

	sprintf(str,"%.2d/%.2d/%.4d",jour,mois,(int)annee);
	DateLabel->setLabel(str);
	sprintf(str,"%.2d:%.2d:%.2d",(int)heure,(int)minute,(int)seconde);
	HourLabel->setLabel(str);
	sprintf(str,"FPS : %4.2f",global.Fps);
	FPSLabel->setLabel(str);
	sprintf(str,"fov=%.3f", global.Fov);
	FOVLabel->setLabel(str);
}


/*****************************************************************************/
// find and select the "nearest" object and retrieve his informations
void findObject(int x, int y)
{	// try to select an object
	GLdouble M[16];	
	GLdouble P[16];
	GLdouble objx[1];
	GLdouble objy[1];
	GLdouble objz[1];
	GLint V[4];
	Switch_to_equatorial();
	// Convert x,y screen pos in 3D vector
	glGetDoublev(GL_MODELVIEW_MATRIX,M);
	glGetDoublev(GL_PROJECTION_MATRIX,P);
	glGetIntegerv(GL_VIEWPORT,V);
	if (gluUnProject(x,global.Y_Resolution-y,1,M,P,V,objx,objy,objz))
	{	global.Pointer[0]=*objx;
		global.Pointer[1]=*objy;
		global.Pointer[2]=*objz;
		if (!SolarSystem.Rechercher(global.Pointer) && global.FlagPlanets)
		{	global.FlagSelect=true;
			SolarSystem.InfoSelect(global.SelectedObject.XYZ, global.SelectedObject.RA,global.SelectedObject.DE, global.SelectedObject.Name, global.SelectedObject.Distance,global.SelectedObject.Size);
			global.Pointer=global.SelectedObject.XYZ;
			RA_en_hms(global.SelectedObject.RAh,global.SelectedObject.RAm,global.SelectedObject.RAs,global.SelectedObject.RA);
			global.SelectedObject.type=2;	//planet type
			global.SelectedObject.RGB[0]=1.0;
			global.SelectedObject.RGB[1]=0.3;
			global.SelectedObject.RGB[2]=0.3;
			global.SelectedObject.Size=global.Y_Resolution*(global.SelectedObject.Size/5);
			InfoSelectLabel->setColour(vec3_t(1.0,0.5,0.5));
		}
		else
		{	if (!messiers.Rechercher(global.Pointer) /*&& FlagNeb*/)
			{	global.FlagSelect=true;
				messiers.InfoSelect(global.SelectedObject.XYZ,global.SelectedObject.RA,global.SelectedObject.DE,global.SelectedObject.Mag,global.SelectedObject.Name,global.SelectedObject.MessierNum,global.SelectedObject.NGCNum,global.SelectedObject.Size);
				global.Pointer=global.SelectedObject.XYZ;
				global.SelectedObject.Size*=0.8;
				RA_en_hms(global.SelectedObject.RAh,global.SelectedObject.RAm,global.SelectedObject.RAs,global.SelectedObject.RA);
				global.SelectedObject.type=1;	//nebulae type
				global.SelectedObject.RGB[0]=0.4;
				global.SelectedObject.RGB[1]=0.5;
				global.SelectedObject.RGB[2]=0.8;
				InfoSelectLabel->setColour(vec3_t(0.4,0.5,0.8));
			}
			else
			{	if (!VouteCeleste.Rechercher(global.Pointer) && global.FlagStars)
				{	global.FlagSelect=true;
					VouteCeleste.InfoSelect(global.SelectedObject.XYZ,global.SelectedObject.RA,global.SelectedObject.DE,global.SelectedObject.Mag,global.SelectedObject.Name,global.SelectedObject.HR,global.SelectedObject.RGB,global.SelectedObject.CommonName);
					global.Pointer=global.SelectedObject.XYZ;
					RA_en_hms(global.SelectedObject.RAh,global.SelectedObject.RAm,global.SelectedObject.RAs,global.SelectedObject.RA);
					global.SelectedObject.type=0;	//star type
					global.SelectedObject.Size=10;
					InfoSelectLabel->setColour(vec3_t(0.4,0.7,0.3));
				}
				else global.FlagSelect=false;
			}
		}
	}
}

/**********************************************************************/
// Update the infos about the selected object in the TextLabel widget
void updateInfoSelectString(void)
{	char objectInfo[300];
	objectInfo[0]=0;
	if (global.SelectedObject.type==0)	//Star
	{	glColor3f(0.4f,0.7f,0.3f);
		sprintf(objectInfo,"Info : %s\nName :%s\nHR  : %.4d\nRA  : %.2dh %.2dm %.2fs\nDE  : %.2fdeg\nMag : %.2f",
			global.SelectedObject.CommonName,
			global.SelectedObject.Name,
			global.SelectedObject.HR,
			global.SelectedObject.RAh,global.SelectedObject.RAm,global.SelectedObject.RAs,
			global.SelectedObject.DE*180/PI,
			global.SelectedObject.Mag);	
	}
	if (global.SelectedObject.type==1)	//Nebulae
	{	glColor3f(0.4f,0.5f,0.8f);
		sprintf(objectInfo,"Name: %s\nNGC : %d\nRA  : %.2dh %.2dm %.2fs\nDE  : %.2fdeg\nMag : %.2f",
			global.SelectedObject.Name,
			global.SelectedObject.NGCNum,
			global.SelectedObject.RAh,global.SelectedObject.RAm,global.SelectedObject.RAs,
			global.SelectedObject.DE*180/PI,
			global.SelectedObject.Mag);		
	}
	if (global.SelectedObject.type==2)	//Planet
	{	glColor3f(1.0f,0.3f,0.3f);
		sprintf(objectInfo,"Name: %s\nRA  : %.2dh %.2dm %.2fs\nDE  : %.2fdeg\nDistance : %.4f AU",
			global.SelectedObject.Name,
			global.SelectedObject.RAh,global.SelectedObject.RAm,global.SelectedObject.RAs,
			global.SelectedObject.DE*180/PI,
			global.SelectedObject.Distance);	
	}
	InfoSelectLabel->setLabel(objectInfo);
	InfoSelectLabel->setVisible(true);
}


/*******************************************************************/
void renderUi()
{	setOrthographicProjection(global.X_Resolution, global.Y_Resolution);	// 2D coordinate
	glPushMatrix();
	glLoadIdentity();

	updateStandardWidgets();

	glEnable(GL_SCISSOR_TEST);
	Base->render(*gc);
	glDisable(GL_SCISSOR_TEST);
	glPopMatrix();
	resetPerspectiveProjection();											// Restore the other coordinate
}

/*******************************************************************************/
void HandleMove(int x, int y)
{	Base->handleMouseMove(x, y);
}

/*******************************************************************************/
void HandleClic(int x, int y, int state, int button)
{	// Convert the name from GLU to my GUI
	enum guiValue bt;
	enum guiValue st;
	switch (button)
	{	case GLUT_RIGHT_BUTTON : bt=GUI_MOUSE_RIGHT; break;
		case GLUT_LEFT_BUTTON : bt=GUI_MOUSE_LEFT; break;
		case GLUT_MIDDLE_BUTTON : bt=GUI_MOUSE_MIDDLE; break;
	}
	if (state==GLUT_UP) st=GUI_UP; else st=GUI_DOWN;

	// Send the mouse event to the User Interface
	if (Base->handleMouseClic(x, y, st, bt))
	{	// If a "unthru" widget was at the clic position 
		return;	
	}
	else
	// Manage the event for the main window
	{	if (state==GLUT_UP) return;
		// Deselect the selected object
		if (button==GLUT_RIGHT_BUTTON)
		{	global.FlagSelect=false;
			InfoSelectLabel->setVisible(false);
			return;
		}
		if (button==GLUT_MIDDLE_BUTTON)
		{	if (global.FlagSelect) Move_To(global.SelectedObject.XYZ);
		}
		if (button==GLUT_LEFT_BUTTON)
		{	// Left or middle clic -> selection of an object
			findObject(x,y);
			// If an object has been found
			if (global.FlagSelect)
			{	updateInfoSelectString();
			}
		}
	}
}

/*******************************************************************************/
void HandleNormalKey(unsigned char key, int state)
{	if (state==GUI_DOWN)
	{	if (global.FlagRealMode)
		{	global.FlagRealMode=false;
			BtRealMode->setActive(false);
			Base->setVisible(true);
		}
		switch(key)
		{	case 'A' :
			case 'a' :	global.FlagConstellationDrawing=!global.FlagConstellationDrawing;
						BtConstellationsDraw->setActive(global.FlagConstellationDrawing);
						break;
			case 'Z' :
			case 'z' :	global.FlagConstellationName=!global.FlagConstellationName;
						BtConstellationsName->setActive(global.FlagConstellationName);
						break;
			case 'E' :
			case 'e' :	global.FlagAzimutalGrid=!global.FlagAzimutalGrid;
						BtAzimutalGrid->setActive(global.FlagAzimutalGrid);
						break;
			case 'R' :
			case 'r' :	global.FlagEquatorialGrid=!global.FlagEquatorialGrid;
						BtEquatorialGrid->setActive(global.FlagEquatorialGrid);
						break;
			case 'T' :
			case 't' :	global.FlagNebula=!global.FlagNebula;
						BtNebula->setActive(global.FlagNebula);
						break;
			case 'Y' :
			case 'y' :	global.FlagGround=!global.FlagGround;
						BtGround->setActive(global.FlagGround);
						break;
			case 'U' :
			case 'u' :	global.FlagFog=!global.FlagFog;
						BtFog->setActive(global.FlagFog);
						break;
			case 'I' :
			case 'i' :	global.FlagRealTime=!global.FlagRealTime;
						BtRealTime->setActive(global.FlagRealTime);
						break;
			case 'O' :
			case 'o' :	global.FlagAcceleredTime=!global.FlagAcceleredTime;
						BtAcceleredTime->setActive(global.FlagAcceleredTime);
						break;
			case 'P' :
			case 'p' :	global.FlagVeryFastTime=!global.FlagVeryFastTime;
						BtVeryFastTime->setActive(global.FlagVeryFastTime);
						break;
			case 'Q' :
			case 'q' :	global.FlagCardinalPoints=!global.FlagCardinalPoints;
						BtCardinalPoints->setActive(global.FlagCardinalPoints);
						break;
			case 'S' :
			case 's' :	global.FlagAtmosphere=!global.FlagAtmosphere;
						BtAtmosphere->setActive(global.FlagAtmosphere);
						break;
			case 'D' :
			case 'd' :	global.FlagRealMode=!global.FlagRealMode;
						BtRealMode->setActive(global.FlagRealMode);
						Base->setVisible(!global.FlagRealMode);
						break;
			case 'H' :
			case 'h' :	global.FlagHelp=!global.FlagHelp;
						BtHelp->setActive(global.FlagHelp);
						HelpWin->setVisible(global.FlagHelp);
						if (global.FlagHelp && global.FlagInfos) global.FlagInfos=false;
						break;
			case 'J' :
			case 'j' :	global.FlagEcliptic=!global.FlagEcliptic;
						break;
			case 'K' :
			case 'k' :	global.FlagEquator=!global.FlagEquator;
						break;
			case 'L' :
			case 'l' :	global.FlagTracking=!global.FlagTracking;
						BtTracking->setActive(global.FlagTracking);
						break;
			case 'M' :
			case 'm' :	global.FlagStars=!global.FlagStars;
						break;

			case '+' :	global.SkyBrightness+=0.1;
						if (global.SkyBrightness>1) global.SkyBrightness=1;
						break;
			case '-' : global.SkyBrightness-=0.1;
						if (global.SkyBrightness<0) global.SkyBrightness=0;
						break;

			case ' ' :	if (global.FlagSelect) Move_To(global.SelectedObject.XYZ);
						break;
/*
			case 'I' :
			case 'i' : global.FlagInfos=!global.FlagInfos; 
						if (global.FlagHelp) global.FlagHelp=false;
						break;
			case 'P' :
			case 'p' : global.FlagPlanets=!global.FlagPlanets; 
						break;*/
		}
	}
}
