/* 
 * Stellarium
 * Copyright (C) 2002 Fabien Chreau
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 */

// Module to manage textures
#include "stellarium.h"
#include "tga.h"

#define NB_TEXTURES 250
#define IMG_TYPE "tga"

static GLuint * textureList;
static char ** textureName;
static GLuint current_tex_num;

void initTextures(void)
{	current_tex_num = 0;
	textureList = new GLuint[NB_TEXTURES];
	textureName = new char * [NB_TEXTURES];
	for (int i=0;i<NB_TEXTURES;i++) textureName[i]=NULL;
	glGenTextures(NB_TEXTURES, textureList);
}

void deleteTextures(void)
{	glDeleteTextures(NB_TEXTURES, textureList);
	for (int i=0;i<NB_TEXTURES;i++) if (textureName[i]!=NULL) delete textureName[i];
	current_tex_num=0;
}

void reloadTextures(void)
{	glDeleteTextures(NB_TEXTURES, textureList);
	glGenTextures(NB_TEXTURES, textureList);
	for (int i=0;i<NB_TEXTURES;i++)
	{	if (textureName[i]!=NULL)
		{	tga_t temp;
			char fullName[128];
			sprintf(fullName,"%s/%s.%s",TEXTURE_DIR,textureName[i],IMG_TYPE);
			if (!temp.Load(fullName, textureList[i]))
			{	printf("Couldn't load texture %s!!\n",fullName);
			}
			current_tex_num=i+1;
		}
	}
		
}

// If the pictures are in 32 bit format --> Alpha layer enabled
unsigned int createTexture(char * texName)
{	tga_t temp;
	char fullName[128];
	current_tex_num++;
	textureName[current_tex_num] = strdup(texName);
	sprintf(fullName,"%s/%s.%s",TEXTURE_DIR,texName,IMG_TYPE);
	if (!temp.Load(fullName, textureList[current_tex_num]))
	{	printf("Couldn't load texture %s!!\n",fullName);
		return 0;
	}
	return textureList[current_tex_num];
}

void deleteTexture(unsigned int i)
{	glDeleteTextures(1,&textureList[i]);
	if (textureName[i]!=NULL)
	{	delete textureName[i];
		textureName[i]=NULL;
	}
	if (i==current_tex_num) current_tex_num--; //should make a full reorganisation
}

int getCurrentTextNum(void)
{	return current_tex_num;
}
